<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_Line;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_LineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\DocumentOrigin;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\SaleDocumentType;
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Create;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Header;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_HeaderFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_LineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Request;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_RequestFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CreateFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CreateResponse;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\ItemIdentifierFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\LocalizedException as LocalizedExceptionAlias;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use RuntimeException;

/**
 * Class Order
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class Order
{
    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var ServiceProvider
     */
    protected $createService;
    /**
     * @var CreateFactory
     */
    protected $createFactory;
    /**
     * @var Create_RequestFactory
     */
    protected $createRequestFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var Create_HeaderFactory
     */
    protected $createHeaderFactory;
    /**
     * @var ArrayOfCreate_LineFactory
     */
    protected $arrayOfCreateLineFactory;
    /**
     * @var Create_LineFactory
     */
    protected $createLineFactory;
    /**
     * @var CartRepositoryInterface
     */
    protected $quoteRepository;
    /**
     * @var Customer
     */
    protected $customerHelper;
    /**
     * @var ItemIdentifierFactory
     */
    protected $itemIdentifierFactory;

    /**
     * Order constructor.
     * @param CredentialProvider $credentialProvider
     * @param CreateFactory $createFactory
     * @param CustomerRepositoryInterface $customerRepository
     * @param Create_HeaderFactory $createHeaderFactory
     * @param RetailContextFactory $retailContextFactory
     * @param Create_RequestFactory $createRequestFactory
     * @param Create_LineFactory $createLineFactory
     * @param ArrayOfCreate_LineFactory $arrayOfCreateLineFactory
     * @param ItemIdentifierFactory $itemIdentifierFactory
     * @param CartRepositoryInterface $quoteRepository
     * @param ServiceProvider $createService
     * @param Customer $customerHelper
     */
    public function __construct(CredentialProvider $credentialProvider,
                                CreateFactory $createFactory,
                                CustomerRepositoryInterface $customerRepository,
                                Create_HeaderFactory $createHeaderFactory,
                                RetailContextFactory $retailContextFactory,
                                Create_RequestFactory $createRequestFactory,
                                Create_LineFactory $createLineFactory,
                                ArrayOfCreate_LineFactory $arrayOfCreateLineFactory,
                                ItemIdentifierFactory $itemIdentifierFactory,
                                CartRepositoryInterface $quoteRepository,
                                ServiceProvider $createService,
                                Customer $customerHelper)
    {
        $this->credentialProvider = $credentialProvider;
        $this->createFactory = $createFactory;
        $this->retailContextFactory = $retailContextFactory;
        $this->createService = $createService;
        $this->createRequestFactory = $createRequestFactory;
        $this->customerRepository = $customerRepository;
        $this->arrayOfCreateLineFactory = $arrayOfCreateLineFactory;
        $this->createHeaderFactory = $createHeaderFactory;
        $this->createLineFactory = $createLineFactory;
        $this->quoteRepository = $quoteRepository;
        $this->customerHelper = $customerHelper;
        $this->itemIdentifierFactory = $itemIdentifierFactory;
    }

    /**
     * @param OrderInterface $order
     * @param $origin
     * @return ArrayOfCreate_Line
     */
    public function getLines(OrderInterface $order, $origin)
    {
        $lines = $this->arrayOfCreateLineFactory->create();

        foreach ($order->getItems() as $item) {
            /*if($item->getProductType() !== 'simple'){
                continue;
            }*/
            $line = $this->createLineFactory->create();

            $identifier = $this->itemIdentifierFactory->create(['reference' => $item->getSku()]);
            $line->setItemIdentifier($identifier);
            $line->setOrigin($origin);
            $line->setNetUnitPrice($item->getPriceInclTax());
            $line->setQuantity($item->getQtyOrdered());
            $line->setUnitPrice($item->getBasePriceInclTax()) ;
            $lines->addToCreate_Line($line);
        }

        return $lines;
    }

    /**
     * @param OrderInterface $order
     * @return CreateResponse
     * @throws LocalizedExceptionAlias
     * @throws NoSuchEntityException
     */
    public function createOrder(OrderInterface $order)
    {
        /** @var Create $createService */
        $createService = $this->createService->create();

        $header = $this->getHeader($order);
        $lines = $this->getLines($order, $header->getOrigin());

        /** @var Create_Request $request */
        $request = $this->createRequestFactory->create([
            'header' => $header
        ]);

        $request->setLines($lines);

        /** @var \FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create $parameters */
        $parameters = $this->createFactory->create();
        $parameters->setCreateRequest($request);
        $parameters->setClientContext($this->getRetailContext());
        if ($createService->Create($parameters) !== false) {
            return $createService->getResult();
        }

        throw new RuntimeException(implode("\n", $createService->getLastError()));
    }

    /**
     * @param OrderInterface $order
     * @return Create_Header
     * @throws LocalizedExceptionAlias
     * @throws NoSuchEntityException
     */
    public function getHeader(OrderInterface $order)
    {
        $isShop = false;

        /** @var Create_Header $header */
        $header = $this->createHeaderFactory->create();
        $customerId = $order->getCustomerId();
        $customerCegidId = null;

        if($customerId === null)
        {
            /* GUEST  */
            $email = $order->getCustomerEmail();
            /** @var \Magento\Customer\Api\Data\CustomerInterface $customer */
            $customer = ObjectManager::getInstance()->create(\Magento\Customer\Api\Data\CustomerInterface::class);
            $customer->setEmail($email);

            $response = $this->customerHelper->searchCustomerByEmail($customer);

            if( $response->getSearchCustomerIdsResult()->getCustomerQueryData() ===null)
            {
                $customer->setLastname($order->getCustomerLastname());
                $customer->setFirstname($order->getCustomerFirstname());

                $customer = $this->customerHelper->createCustomer($customer);
                $customerCegidId = $customer->getData('cegid_id');
            }else{

             //   $customer->setLastname($order->getCustomerLastname());
         //       $customer->setFirstname($order->getCustomerFirstname());

            //    $customer = $this->customerHelper->createCustomer($customer);
            //    $customerCegidId = $customer->getData('cegid_id');
                $customerCegidId =   $response->getSearchCustomerIdsResult()->getCustomerQueryData()[0]->getCustomerId();
            }

        } else {

            $customer = $this->customerRepository->getById($order->getCustomerId());
            $cegidCustomerId = $customer->getCustomAttribute('cegid_id');
            if (!$cegidCustomerId) {
                $response = $this->customerHelper->searchCustomerByEmail($customer);

                if($response ===null)
                {
                    $customer->setLastname($order->getCustomerLastname());
                    $customer->setFirstname($order->getCustomerFirstname());

                 $customer=   $this->customerHelper->createCustomer($customer);
                    $customerCegidId = $customer->getData('cegid_id');
                }else{
                    $customerCegidId =   $response->getSearchCustomerIdsResult()->getCustomerQueryData()[0]->getCustomerId();
                }
            }else{
                $customerCegidId = $customer->getCustomAttribute('cegid_id');

            }

        }

        $header->setCurrencyId('USD');
        $header->setCustomerId($customerCegidId);
        $header->setInternalReference($order->getIncrementId());
        $header->setExternalReference($order->getIncrementId());
        $header->setDate(substr($order->getUpdatedAt(), 0, 10));

        $quote = $this->quoteRepository->get($order->getQuoteId());
        $extensionAttributes = $quote->getExtensionAttributes();
        if ($extensionAttributes !== null) {
            $isShop = true;
           // $sellerId = $extensionAttributes;
           //$header->setSalesPersonId($sellerId = 'ITD240076');
        }

        $isShop = true;
        $header->setStoreId('ECO01'); // ITD24

        $header->setTaxExcluded(false);
        $header->setType(SaleDocumentType::VALUE_RECEIPT);
        $header->setOrigin($isShop ? DocumentOrigin::VALUE_SHOP : DocumentOrigin::VALUE_ECOMMERCE);

        // Set Ominchannel

        //$header->setOmniChannel();

        return $header;
    }

    /**
     * @return RetailContext
     */
    public function getRetailContext()
    {
        return $this->retailContextFactory->create(['databaseId' =>
            $this->credentialProvider->getDatabase('sale_document_service')
        ]);
    }
}