<?php

namespace FiloBlu\Cegid\Observer;

use Exception;
use FiloBlu\Cegid\Helper\Cegid\Customer;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Psr\Log\LoggerInterface;

/**
 * Class CustomerRegister
 * @package FiloBlu\Cegid\Observer
 */
class CustomerRegister implements ObserverInterface
{
    /**
     * @var Customer
     */
    protected $customerHelper;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * CustomerRegister constructor.
     * @param CustomerRepositoryInterface $customerRepository
     * @param Customer $customerHelper
     * @param LoggerInterface $logger
     */
    public function __construct(
        CustomerRepositoryInterface $customerRepository,
        Customer $customerHelper,
        LoggerInterface $logger)
    {
        $this->customerHelper = $customerHelper;
        $this->customerRepository = $customerRepository;
        $this->logger = $logger;
    }

    /**
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        $customerData = $observer->getData('customer');

        if (!$customerData || !$customerData->getId()) {
            return;
        }

        try {

            $customer = $this->customerRepository->getById($customerData->getId());
            $result = $this->customerHelper->searchCustomerByEmail($customer);
            $response = $result->getSearchCustomerIdsResult()->getCustomerQueryData();

            if($response === null){
                /** Create customer on Cegid */
                $this->customerHelper->createCustomer($customer);
            }else{
                /** Assign Cegid Id to our customer */
                $customer->setCustomAttribute('cegid_id', $response[0]->getCustomerId());
               $this->customerRepository->save($customer);
            }

        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        }
    }
}